/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.mcsla.core.metrics;

import eu.DECIDEh2020.appManager.models.Parameter;

import java.util.List;
import java.util.stream.Collectors;

class AvailabilityMTBFMTTR implements PredefinedExpression<Double, Long> {

    @Override
    public Double calculate(List<Long> input, List<Parameter> parameters) {
        Double uptimeBegin;
        Double downtimeBegin;
        Double uptimeAgain;
        if (input.size() == 3) {
            uptimeBegin = input.get(0).doubleValue();
            downtimeBegin = input.get(1).doubleValue();
            uptimeAgain = input.get(2).doubleValue();
        } else {
            uptimeBegin = 0.0; // or restore it
            downtimeBegin = input.get(0).doubleValue();
            uptimeAgain = input.get(1).doubleValue();
        }

        Double mtbf = downtimeBegin - uptimeBegin;
        Double mttr = uptimeAgain - downtimeBegin;
        return 100.0 * (mtbf/(mtbf + mttr));
    }

    @Override
    public Double calculateFromStrings(List<String> input, List<Parameter> parameters) {
        return calculate(input.stream().map(Long::valueOf).collect(Collectors.toList()), parameters);
    }

}
