/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.mcsla.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.models.Sla;
import eu.DECIDEh2020.mcsla.service.repositories.SlaFactory;
import io.vertx.core.Vertx;
import io.vertx.junit5.Checkpoint;
import io.vertx.junit5.VertxExtension;
import io.vertx.junit5.VertxTestContext;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;

import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Path;

@DisplayName("Testing the MCSLA Service")
@ExtendWith(VertxExtension.class)
public class MCSLAServiceTest {

    @Test
    @DisplayName("First test")
    void deployService(Vertx vertx, VertxTestContext testContext) {
        Checkpoint deployment = testContext.checkpoint();
        vertx.deployVerticle(new MainVerticle(), testContext.succeeding(id -> deployment.flag()));
    }

    @Test
    @DisplayName("Test for SlaFactory")
    void slaFactory(Vertx vertx, VertxTestContext testContext) {
        Path path = FileSystems.getDefault().getPath("src/test/resources/DECIDE.json");
        try {
            AppDescription appDescription = new ObjectMapper().readValue(path.toFile(), AppDescription.class);
            Sla sla = new SlaFactory().createFromNfrs(appDescription.getNfrs(), appDescription.getMicroservices());
            new ObjectMapper().writerWithDefaultPrettyPrinter().writeValue(System.out, sla);
            testContext.completeNow();
        } catch (IOException e) {
            testContext.failNow(e);
        }
    }

}
