import React, { Component } from "react";
import { connect } from 'react-redux'
import { withStyles } from "@material-ui/core/styles/index";
import Typography from "@material-ui/core/Typography";
import ExpansionPanel from '@material-ui/core/ExpansionPanel';
import Paper from '@material-ui/core/Paper';
import ExpansionPanelSummary from '@material-ui/core/ExpansionPanelSummary';
import ExpansionPanelDetails from '@material-ui/core/ExpansionPanelDetails';
import ExpansionPanelActions from '@material-ui/core/ExpansionPanelActions';
import ExpandMoreIcon from '@material-ui/icons/ExpandMore';
import TextField from '@material-ui/core/TextField';
import IconButton from '@material-ui/core/IconButton';
import DeleteIcon from '@material-ui/icons/Delete';
import Avatar from '@material-ui/core/Avatar';
import UpdateIcon from '@material-ui/icons/Update';
import NetworkCheckIcon from '@material-ui/icons/NetworkCheck';
import LocationIcon from '@material-ui/icons/LocationOn';
import EuroSymbolIcon from '@material-ui/icons/EuroSymbol';
import BlurOnIcon from '@material-ui/icons/BlurOn';
import QuestioAnswerIcon from '@material-ui/icons/QuestionAnswer';
import Grid from '@material-ui/core/Grid';
import ServiceObjectiveAvailability from './ServiceObjectiveAvailability';
import PerformanceManually from './PerformanceManually';
import ScalabilityManually from './ScalabilityManually';
import LocationManually from './LocationManually';
import CostManually from './CostManually';

const styles = theme => ({
    column: {
        flexBasis: '20%',
        marginLeft: theme.spacing.unit
    },
    flex: {
        flex: 1
    },
    button: {
        margin: theme.spacing.unit,
    },
    formControl: {
        margin: theme.spacing.unit * 3,
    },
    papers: {
        ...theme.mixins.gutters(),
        padding: theme.spacing.unit * 2
      }
});

const units = {
    "percentage": "%",
    "millisecond": "ms",
    "request/second": "req/s",
    "euro": "€"
}

class ServiceObjective extends Component {

    constructor(props) {
        super(props);
        let icon;
        let manually;
        switch (props.objective.termName) {
        case "Availability":
            icon = <UpdateIcon/>;
            manually = <ServiceObjectiveAvailability objective={props.objective} />;
            break;
        case "Performance":
            icon = <NetworkCheckIcon/>;
            manually = <PerformanceManually objective={props.objective} />;
            break;
        case "Scalability":
            icon = <BlurOnIcon/>;
            manually = <ScalabilityManually objective={props.objective} />;
            break;
        case "Location":
        case "Region":
        case "Zone":
            icon = <LocationIcon/>;
            manually = <LocationManually objective={props.objective} />;
            break;
        case "Cost":
            icon = <EuroSymbolIcon/>;
            manually = <CostManually objective={props.objective} />;
            break;
        case "Legal level":
            icon = "§";
            break;
        case "Provider":
            icon = "P";
            break;
        default:
            icon = <QuestioAnswerIcon/>;
            manually = <div />;
            break;
        }
        this.state = {
            comment: props.objective.comment,
            icon: icon,
            manually: manually
        };
    }

    handleComment(event) {
        this.setState({comment: event.target.value})
    }

    handleDefinition(event) {
        this.setState({definition: event.target.value})
    }

    render() {
        const { objective, readonly, classes, onClick } = this.props;
        const { icon } = this.state;

        return (
            <ExpansionPanel>
                <ExpansionPanelSummary expandIcon={<ExpandMoreIcon />}>
                    <Grid container spacing={24} alignItems="center">
                        <Grid item>
                            <Avatar>{icon}</Avatar>
                        </Grid>
                        <Grid item>
                            <Typography variant="subheading">{objective.termName}</Typography>
                        </Grid>
                        {readonly ?  
                            <Grid item>
                                <Typography color="textSecondary" variant="subheading">{objective.comment}</Typography>
                            </Grid> : null
                        }
                    </Grid>
                </ExpansionPanelSummary>
                <ExpansionPanelDetails>
                    <Grid container spacing={24} column>
                        {readonly ? null : <Grid item sm={12}><TextField label="Comment" fullWidth value={this.state.comment} onChange={(event) => this.handleComment(event)} /></Grid>}
                        <Grid item>
                            <Typography variant="h4">{objective.value + (objective.unit ? ' ' + units[objective.unit] : '')}</Typography>
                        </Grid>
                        {objective.metrics && objective.metrics.length > 0 ?
                            <Grid item sm={12} container>
                                <Grid item sm={12}>
                                    <Typography color="textSecondary" variant="h5" gutterBottom>Metrics</Typography>
                                </Grid>
                                <Grid item sm={12}>
                                    <Paper className={classes.papers}>
                                    {readonly ? 
                                        <div>
                                            <Typography variant="subheading">{objective.metrics[0].descriptor}</Typography>
                                            <Typography variant="body1">{objective.metrics[0].expression.note}</Typography>
                                        </div>
                                        : this.state.manually }
                                    </Paper>
                                </Grid>
                            </Grid> : null
                        }
                    </Grid>
                </ExpansionPanelDetails>
                {readonly ? null :
                    <ExpansionPanelActions>
                        <IconButton className={classes.button} onClick={onClick}>
                            <DeleteIcon />
                        </IconButton>
                    </ExpansionPanelActions>
                }
            </ExpansionPanel>
        )
    }
}

export default connect(null, null)(withStyles(styles, {withTheme: true})(ServiceObjective))