import React from 'react';
import ReactDOM from 'react-dom';
import { Provider } from 'react-redux';
import { createStore, applyMiddleware, combineReducers } from 'redux';
import createSagaMiddleware from 'redux-saga';
import createHistory from 'history/createBrowserHistory';
import { ConnectedRouter, routerReducer, routerMiddleware } from 'react-router-redux';

import './index.css';

import App from './components/App';
import registerServiceWorker from './registerServiceWorker';

import { repository, aggregations, decideApp, app } from './reducers/mcslauiApp';

import rootSaga from './sagas/saga'
import { fetchServiceUriRequest } from './actions/repository';

const history = createHistory();
const middleware = routerMiddleware(history);

const sagaMiddleware = createSagaMiddleware();
const store = createStore(
    combineReducers({
        repository,
        aggregations,
        decideApp,
        app,
        router: routerReducer
    }),
    applyMiddleware(middleware, sagaMiddleware)
);
sagaMiddleware.run(rootSaga);

store.dispatch(fetchServiceUriRequest('/app-conf.json'));

ReactDOM.render(
    <Provider store={store}>
        <ConnectedRouter history={history}>
            <App />
        </ConnectedRouter>
    </Provider>,
    document.getElementById('root')
);
registerServiceWorker();
