import {
    USE_REPOSITORY,
    FETCH_SERVICE_URI_REQUEST,
    FETCH_SERVICE_URI_RESPONSE,
    FETCH_SERVICE_URI_FAILURE,
    RESET_REPOSITORY_REQUEST,
    RESET_REPOSITORY_RESPONSE,
    RESET_REPOSITORY_FAILURE } from '../actions/repository';

 import {
    FETCH_DECIDE_APP_REQUEST,
    FETCH_DECIDE_APP_FAILURE,
    FETCH_DECIDE_APP_RESPONSE,
    AGGREGATE_OBJECTIVES_REQUEST,
    AGGREGATE_OBJECTIVES_RESPONSE,
    AGGREGATE_OBJECTIVES_FAILURE } from "../actions/mcsla";
import {
    SET_SLA_DESCRIPTION,
    SET_SLA_VALIDITY,
    SET_SLA_VISIBILITY,
    ADD_COVERED_SERVICE,
    DELETE_COVERED_SERVICE,
    ADD_OBJECTIVE,
    DELETE_OBJECTIVE } from '../actions/editmcsla';

import { RESET_ERROR } from '../actions/app';

const initialRepository = {
    uri: "",
    token: "",
    username: "",
    password: "",
    status: "unknown",
};

const initialDecideApp = {

};

const initialAggregations = {
    Availability: {},
    Performance: {},
    Scalability: {},
    Location: {},
    Cost: {}
}

const initialApp = {
    serviceUri: "http://mcsla-service:8080",
    isFetching: false,
    appName: "",
    error: {
        message: ""
    }
};

const objectiveTemplates = {
    Availability: {
        "termName": "Availability",
        "type": "slo",
        "value": 0,
        "unit": "percentage",
        "conditionStatement": "greaterOrEqual",
        "metrics": [
            {
                "id": "CSA_001",
                "descriptor": "Uptime during a time period",
                "scale": "ratio",
                "expression": {
                    "expression": "AVAILABILITY_AGGREGATION_SUMTYPE",
                    "expressionLanguage": "predefined",
                    "unit": "percentage"
                }
            }
        ]
    },
    Performance: {
        "termName": "Performance",
        "type": "slo",
        "value": 0,
        "unit": "millisecond",
        "conditionStatement": "lessOrEqual",
        "metrics": [
            {
                "id": "CSP_001",
                "descriptor": "Response time for a request",
                "scale": "ratio",
                "expression": {
                    "expression": "AGGREGATION_MAXTYPE",
                    "expressionLanguage": "predefined",
                    "unit": "millisecond"
                }
            }
        ]
    },
    Scalability: {
        "termName": "Scalability",
        "type": "sqo",
        "value": "",
        "conditionStatement": "equal",
        "metrics": [
            {
                "id": "CSS_001",
                "descriptor": "Scale-up for performance",
                "scale": "ordinal",
                "expression": {
                    "expression": "EMPTY",
                    "expressionLanguage": "predefined"
                }
            }
        ]
    },
    Location: {
        "termName": "Location",
        "type": "sqo",
        "value": "",
        "unit": "english",
        "conditionStatement": "equal",
        "metrics": [
            {
                "id": "CSL_1",
                "descriptor": "Location based on continents and countries",
                "scale": "nominal",
                "expression": {
                    "expression": "EMPTY",
                    "expressionLanguage": "predefined"
                }
            }
        ]
    },
    Cost: {
        "termName": "Cost",
        "type": "slo",
        "value": 0,
        "unit": "euro",
        "conditionStatement": "less",
        "metrics": [
            {
                "id": "CSC_1",
                "descriptor": "Cost limitation",
                "scale": "ratio",
                "expression": {
                    "expression": "AGGREGATION_SUMTYPE",
                    "expressionLanguage": "predefined",
                    "unit": "euro"
                }
            }
        ]
    }
}

export function repository(state = initialRepository, action) {
    switch (action.type) {
        case USE_REPOSITORY:
            return {
                ...state,
                uri: action.uri,
                token: action.token,
                username: action.username,
                password: action.password
            };
        case FETCH_SERVICE_URI_RESPONSE:
            const params = getParams();
            return {
                ...state,
                uri: params.uri,
                token: params.token,
                username: params.username,
                password: params.password
            };
        default:
            return state
    }
}

export function aggregations(state = initialAggregations, action) {
    switch (action.type) {
        case AGGREGATE_OBJECTIVES_RESPONSE:
            const newState = {
                ...state
            };
            newState[action.termName] = {
                [action.predefined]: action.aggregated
            };
            return newState;
        default: return state;
    }
}

export function decideApp(state = initialDecideApp, action) {
    switch (action.type) {
        case FETCH_DECIDE_APP_RESPONSE:
            return { ...action.app }; 
        case SET_SLA_DESCRIPTION:
            return {
                ...state,
                mcsla: {
                    ...state.mcsla,
                    sla: {
                        ...state.mcsla.sla,
                        description: action.decsription
                    }
                }
            };
        case SET_SLA_VALIDITY:
            return {
                ...state,
                mcsla: {
                    ...state.mcsla,
                    sla: {
                        ...state.mcsla.sla,
                        validityPeriod: action.validity
                    }
                }
            };
        case SET_SLA_VISIBILITY:
            return {
                ...state,
                mcsla: {
                    ...state.mcsla,
                    sla: {
                        ...state.mcsla.sla,
                        visibility: action.visibility
                    }
                }
            };
        case ADD_COVERED_SERVICE:
            let coveredServices = state.mcsla.sla.coveredServices;
            if (!coveredServices) {
                coveredServices = []
            }
            coveredServices.push(action.service);
            return {
                ...state,
                mcsla: {
                    ...state.mcsla,
                    sla: {
                        ...state.mcsla.sla,
                        coveredServices: coveredServices
                    }
                }
            };
        case DELETE_COVERED_SERVICE:
            const reducedServices = state.mcsla.sla.coveredServices.filter(s => s !== action.service);
            return {
                ...state,
                mcsla: {
                    ...state.mcsla,
                    sla: {
                        ...state.mcsla.sla,
                        coveredServices: reducedServices
                    }
                }
            };
        case ADD_OBJECTIVE:
            const newObjectives = state.mcsla.sla.objectives ? state.mcsla.sla.objectives : [];
            newObjectives.unshift(objectiveTemplates[action.termName]);
            return {
                ...state,
                mcsla: {
                    ...state.mcsla,
                    sla: {
                        ...state.mcsla.sla,
                        objectives: newObjectives
                    }
                }
            };
        case DELETE_OBJECTIVE:
            const objectives = state.mcsla.sla.objectives.filter(s => s.termName !== action.termName);
            return {
                ...state,
                mcsla: {
                    ...state.mcsla,
                    sla: {
                        ...state.mcsla.sla,
                        objectives: objectives
                    }
                }
            };
        case RESET_REPOSITORY_RESPONSE:
            return {
                ...action.response
            }
        default:
            return state
    }
}

export function app(state = initialApp, action) {
    switch (action.type) {
        case RESET_ERROR:
            return {
                ...state,
                error: {message: ""}
            }
        case FETCH_SERVICE_URI_REQUEST:
            return {
                ...state,
                isFetching: true
            };
        case FETCH_SERVICE_URI_RESPONSE:
            return {
                ...state,
                ...action.uri,
                isFetching: false
            };
        case FETCH_SERVICE_URI_FAILURE:
            return {
                ...state,
                isFetching: false,
                error: action.error
            };
        case FETCH_DECIDE_APP_REQUEST:
            return {
                ...state,
                isFetching: true
            };
        case FETCH_DECIDE_APP_RESPONSE:
            return {
                ...state,
                appName: action.name,
                isFetching: false
            };
        case FETCH_DECIDE_APP_FAILURE:
            return {
                ...state,
                isFetching: false,
                error: action.error
            };
        case AGGREGATE_OBJECTIVES_REQUEST:
            return {
                ...state,
                isFetching: true
            };
        case AGGREGATE_OBJECTIVES_RESPONSE:
            return {
                ...state,
                isFetching: false
            };
        case AGGREGATE_OBJECTIVES_FAILURE:
            return {
                ...state,
                isFetching: false,
                error: action.error
            };
        case RESET_REPOSITORY_REQUEST:
            return {
                ...state,
                isFetching: true
            };
        case RESET_REPOSITORY_RESPONSE:
            return {
                ...state,
                isFetching: false
            };
        case RESET_REPOSITORY_FAILURE:
            return {
                ...state,
                isFetching: false,
                error: action.error
            };
        default:
                return state
    }
}

function getParams() {
    let query = window.location.search.substr(1);
    let result = {};
    query.split("&").forEach(function(part) {
        let item = part.split("=");
        result[item[0]] = decodeURIComponent(item[1]);
    });
    return result;
}
